'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = createMigration;

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _constants = require('redux-persist/constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var processKey = function processKey(key) {
  var int = parseInt(key);
  if (isNaN(int)) throw new Error('redux-persist-migrate: migrations must be keyed with integer values');
  return int;
};

function createMigration(manifest, _ref) {
  var selector = _ref.selector,
      setter = _ref.setter,
      _ref$semver = _ref.semver,
      semver = _ref$semver === undefined ? false : _ref$semver;

  if (typeof selector === 'string') {
    var reducerKey = selector;
    selector = function selector(state) {
      return state && state[reducerKey] && state[reducerKey].version;
    };
    setter = function setter(state, version) {
      if (['undefined', 'object'].indexOf(_typeof(state[reducerKey])) === -1) {
        console.error('redux-persist-migrate: state for setter key must be an object or undefined');
        return state;
      }
      state[reducerKey] = state[reducerKey] || {};
      state[reducerKey].version = version;
      return state;
    };
  }

  var versionKeys = !semver ? Object.keys(manifest).map(processKey).sort() : _semver2.default.sort(Object.keys(manifest));
  var currentVersion = versionKeys[versionKeys.length - 1];

  var migrationDispatch = function migrationDispatch(next) {
    return function (action) {
      if (action.type === _constants.REHYDRATE) {
        var incomingState = action.payload;
        var incomingVersion = selector(incomingState);
        if (incomingVersion !== currentVersion) {
          var migratedState = migrate(incomingState, incomingVersion);
          action.payload = migratedState;
        }
      }
      return next(action);
    };
  };

  var migrate = function migrate(state, version) {
    versionKeys.filter(function (v) {
      return !version || (semver ? !_semver2.default.valid(version) || _semver2.default.gt(v, version) : v > version);
    }).forEach(function (v) {
      state = manifest[v](state);
    });
    state = setter(state, currentVersion);
    return state;
  };

  return function (next) {
    return function (reducer, initialState, enhancer) {
      var store = next(reducer, initialState, enhancer);
      return _extends({}, store, {
        dispatch: migrationDispatch(store.dispatch)
      });
    };
  };
}